(function(env) {
// Mostly adapted from Cordova iOS bridge code

console.log("I have been injected");

var bmnative = env.bmnative,
    callbacks = {},
    unhandledMessages = {};


var addCallback = function(name, cb) {
  if(callbacks[name] === undefined) {
    callbacks[name] = [];
  }

  callbacks[name].push(cb);

  var messages = unhandledMessages[name];
  if(messages !== undefined) {
    for(var i = 0; i < messages.length; ++i) {
      cb.apply(null, messages[i]);
    }
    unhandledMessages[name] = undefined;
  }
};

var handleMessageFromNative = function(name, args) {
  console.log(name, args);
  var cb = callbacks[name];

  if(cb !== undefined) {
    for(var i = 0; i < cb.length; ++i) {
      try {
    	  cb[i].apply(null, args);
      } catch(e) {
    	  console.error("error running callback", e);
      }
    }
  } else {
    if(unhandledMessages[name] === undefined) {
      unhandledMessages[name] = [];
    }
    unhandledMessages[name].push(args);
  }
};

function pollNativeMessages(fromOnline) {
	console.log("polling for messages")
	var encoded = bmnative.pollMessages(fromOnline || false);
	if(encoded) {
		console.log(encoded);
		var messages = JSON.parse(encoded);
		for(var i = 0; i < messages.length; ++i) {
			var message = messages[i];
			handleMessageFromNative(message[0], message[1]);
		}
	}
}

function pollNativeMessagesFromOnline() {
	pollNativeMessages(true);
}

window.addEventListener('online', pollNativeMessagesFromOnline, false);
window.addEventListener('offline', pollNativeMessagesFromOnline, false);

function encodeParameter(param) {
  if(typeof param === 'undefined') {
    throw "undefined is not valid parameter";
  }
  else if(param instanceof Object) {
    // if non-scalar, use json encoded string
    return JSON.stringify(param);
  }
  else if(param === null) {
    // null is represented by the empty string
    return "";
  }
  else {
    return param;
  }
}

function sendToHost(messageName) {
  // TODO: consider using params inline
  var params = Array.prototype.splice.call(arguments, 1);
  for(var i = 0; i < params.length; ++i) {
    params[i] = encodeParameter(params[i]);
  }
  bmnative.sendToHost(messageName, params.length, JSON.stringify(params));
}

env.sendToHost = sendToHost;
env.addCallback = addCallback;

env.sendButton = function(button, state) {
	bmnative.sendButton(button, state);
};

var startTime;

addCallback("latencyReply", function() {
  var elapsed = new Date() - startTime;
  console.log(elapsed);
});

env.testLatency = function() {
  startTime = new Date();
  bmnative.testLatency();
};

/// TODO: is this the right timing?
env.setTimeout(pollNativeMessages, 0);
env.setTimeout(function(){bmnative.onBridgeReady()}, 0);

})(window);